<?php
/**
 * @var string $action "process_record_file_path"
 * @var array  $params
 *
 * Подробное описание в документации Itgrix:
 * https://docs.itgrix.ru/custom_bx
 *
 * Вывод в лог можно сделать так:
 *   $this->logDebug($message);
 * Функции для вывода с другим уровнем логирования: logInfo, logWarn, logError.
 *
 * Запросы в Битрикс24 реализованы в классе Bitrix (lib/crm.php)
 * Подробности и примеры в документации Itgrix:
 * https://docs.itgrix.ru/custom_bx/zaprosy-v-bitriks-iz-kastomizacii
 *
 * Дла запросов в БД Asterisk предусмотрены функции (lib/Utils.php):
 *   Utils::mysqliQuery($sql, $getResult);   - запрос через php-mysqli
 *   Utils::mysqlClcQuery($sql, $getResult); - запрос через MySQL Command-Line Client
 */

$callId = $params['call_id'];
$from   = $params['from'];
$to     = $params['to'];
$path   = $params['path'];

/*  Удаление дублей в пути
    /var/spool/asterisk/monitor/2010/10/10/2010/10/10/recording.wav → /var/spool/asterisk/monitor/2010/10/10/recording.wav
    /var/spool/asterisk/monitor/2010/10/10//2010/10/10/recording.wav → /var/spool/asterisk/monitor/2010/10/10/recording.wav
    /var/spool/asterisk/monitor/2010/10/10//var/spool/asterisk/monitor/2010/10/10/recording.wav → /var/spool/asterisk/monitor/2010/10/10/recording.wav
    /var/spool/asterisk/monitor/2010/10/10/var/spool/asterisk/monitor/2010/10/10/recording.wav → /var/spool/asterisk/monitor/2010/10/10/recording.wav
    /var/spool/asterisk/monitor//var/spool/asterisk/monitor/recording.wav → /var/spool/asterisk/monitor/recording.wav
    /var/spool/asterisk/monitor/var/spool/asterisk/monitor/recording.wav → /var/spool/asterisk/monitor/recording.wav
*/
// $path = preg_replace('/((\\/\\d+){3}\\/?)\\1+/', '$1', $path);
// $path = preg_replace('/^(.+)\\1+(\\/?)([^\\/]+)$/', '$1$2$3', $path);

//
// Пример конвертации записей разговоров в mp3
//

$updateFilenameInCdr = false; // Заменить в CDR имя файла на mp3
$removeWavFile       = false; // Удалить исходный wav-файл

$recPathInfo = pathinfo($path);
$recBasename  = $recPathInfo['basename'];
$recExtension = $recPathInfo['extension'];

if (strtolower($recExtension) == 'wav') {

    $newPath = Utils::convertToMp3($path);
    if ($newPath === false) {
        return array(
            'state' => 'error',
            'data' => "Failed to convert recording file to mp3: '$path'",
        );
    }
    $this->logInfo("Converted recording file: '$newPath'");

    $newBasename = basename($newPath);

    // Заменить в CDR имя файла на новое (wav на mp3)
    if ($updateFilenameInCdr) {
        $dbResult = Utils::mysqliQuery(
            "UPDATE `cdr` SET `recordingfile` = '{$newBasename}'" .
            " WHERE `recordingfile` = '{$recBasename}'");

        if ($dbResult === false) {
            return array(
                'state' => 'error',
                'data' => 'Ошибка при запросе в CDR',
            );
        }
        $this->logInfo("Updated recording filename in CDR: '$newBasename'");
    }

    // Удалить исходный файл с расширением .wav
    if ($removeWavFile) {
        $rmSuccess = unlink($path);
        if (! $rmSuccess) {
            return array(
                'state' => 'error',
                'data' => "Не удалось удалить исходный wav-файл: '$path'",
            );
        }
        $this->logInfo("Removed recording file: '$path'");
    }

    $path = $newPath;
}

return array(
    'state' => 'success',
    'data'  => array(
        'path' => $path,
    ),
);
