#!/usr/bin/env php
<?php
// version: 3.11.10

require_once  __DIR__.'/lib/Customizer.php';

define('EXIT_CODE_SUCCESS', 0);
define('EXIT_CODE_INVALID_ARGS', 1);
define('EXIT_CODE_FAILED_TO_DECODE_PARAMS', 2);
define('EXIT_CODE_EMPTY_DECODED_PARAMS', 3);
define('EXIT_CODE_INVALID_JSON_PARAMS', 4);

if ($argc !== 3) {
    exit(EXIT_CODE_INVALID_ARGS);
}

$action = $argv[1];
$paramsStr = $argv[2];

$paramsJson = $paramsStr;

if (!empty($paramsStr) && $paramsStr[0] != '{') { // not a JSON
    exec(sprintf("bash -c \"set -o pipefail && echo '%s' | base64 --decode | gzip -d\"", $paramsStr),
        $output, $resultCode);
    if ($resultCode != 0) {
        fprintf(STDERR, "failed to decode gzip-base64 params, exit code: %d", $resultCode);
        exit(EXIT_CODE_FAILED_TO_DECODE_PARAMS);
    }
    if (empty($output)) {
        fprintf(STDERR, "empty decoded gzip-base64 params");
        exit(EXIT_CODE_EMPTY_DECODED_PARAMS);
    }
    $paramsJson = $output[0];
}

$params = json_decode($paramsJson, true);
if (json_last_error() !== JSON_ERROR_NONE) {
    fprintf(STDERR,
        "failed to decode JSON params, last error: %d", json_last_error());
    exit(EXIT_CODE_INVALID_JSON_PARAMS);
}

$result = array();

try {
    $customizer = new Customizer($action, $params);
    $result = $customizer->run();
} catch (Exception $e) {
    $result = array(
        'state' => 'error',
        'data'  => array(
            'message' => $e->getMessage(),
            'trace'   => $e->getTrace(),
        ),
    );
}

echo json_encode($result), PHP_EOL;
exit(EXIT_CODE_SUCCESS);
