<?php

define('SETTINGS_PATH', realpath(__DIR__.'/../settings.json'));

/**
 * Class Settings
 *
 * Настройки кастомизатора.
 *
 * ВАЖНО! При использовании нескольких копий модуля Itgrix нужно копировать
 * папку customizer для каждой копии (customizer, customizer-1, customizer-2 и т.д.)
 * и прописывать в settings.json свои пути для каждой копии.
 */
class Settings {

    /**
     * Получить название поля CRM в конфиге модуля Itgrix
     *
     * @return string Путь к файлу
     *
     * @throws Exception В случае ошибки
     */
    public static function getConfigCrmField() {
        if (! isset($GLOBALS['configCrmField'])) {
            self::readAll();
        }
        return $GLOBALS['configCrmField'];
    }

    /**
     * Получить путь к конфигурационному файлу модуля Itgrix
     *
     * @return string Путь к файлу
     *
     * @throws Exception В случае ошибки
     */
    public static function getConfigPath() {
        if (! isset($GLOBALS['configPath'])) {
            self::readAll();
        }
        return $GLOBALS['configPath'];
    }

    /**
     * Получить путь к файлу лога модуля Itgrix
     *
     * @return string Путь к файлу
     *
     * @throws Exception В случае ошибки
     */
    public static function getLogPath() {
        if (! isset($GLOBALS['logPath'])) {
            self::readAll();
        }
        return $GLOBALS['logPath'];
    }

    /**
     * Получить таймаут запросов
     *
     * @return integer Таймаут в секундах
     *
     * @throws Exception В случае ошибки
     */
    public static function getRequestTimeoutSec() {
        if (! isset($GLOBALS['requestTimeoutSec'])) {
            self::readAll();
        }
        return $GLOBALS['requestTimeoutSec'];
    }

//==============================================================================

    /**
     * Прочитать все необходимые настройки из файла
     *
     * Читает поля configCrmField, configPath, logPath, requestTimeoutSec
     * из файла SETTINGS_PATH
     * и сохраняет в одноименные поля $GLOBALS.
     *
     * @throws Exception В случае ошибки
     */
    protected static function readAll() {
        $contents = file_get_contents(SETTINGS_PATH);
        if ($contents === false) {
            throw new Exception('Failed to read settings from file: '.SETTINGS_PATH);
        }

        $settings = json_decode($contents, true);
        if ($settings === null) {
            throw new Exception('Failed to parse JSON from settings file: '.SETTINGS_PATH);
        }

        $fields = array(
            array('name' => 'configCrmField',    'type' => 'string'),
            array('name' => 'configPath',        'type' => 'string'),
            array('name' => 'logPath',           'type' => 'string'),
            array('name' => 'requestTimeoutSec', 'type' => 'integer'),
        );

        foreach ($fields as $field) {
            if (! isset($settings[$field['name']])) {
                throw new Exception(sprintf("No field '%s' in settings file '%s'",
                    $field['name'], SETTINGS_PATH));
            }

            $type = gettype($settings[$field['name']]);
            if ($type !== $field['type']) {
                throw new Exception(sprintf(
                    "Invalid type of field '%s' (expected '%s' but got '%s') in settings file '%s'",
                    $field['name'], $field['type'], $type, SETTINGS_PATH));
            }

            $GLOBALS[$field['name']] = $settings[$field['name']];
        }
    }
}
