#!/usr/bin/env bash

jGet() {
  local file="$1"
  local addr="$2"

  "${JEDIT_BIN}" \
      --file="${file}" \
      --action=get \
      --addr="${addr}"
}

jSet() {
  local file="$1"
  local addr="$2"
  local defval="$3"

  "${JEDIT_BIN}" \
      --file="${file}" \
      --action=set \
      --addr="${addr}" \
      --defval="${defval}"
}

jDel() {
  local file="$1"
  local addr="$2"

  "${JEDIT_BIN}" \
      --file="${file}" \
      --action=delete \
      --addr="${addr}"
}

jAdd() {
  local file="$1"
  local addr="$2"
  local defval="$3"

  "${JEDIT_BIN}" \
      --file="${file}" \
      --action=add \
      --addr="${addr}" \
      --defval="${defval}"
}

jMove() {
  local file="$1"
  local addr="$2"
  local newaddr="$3"
  local defval="$4"

  "${JEDIT_BIN}" \
      --file="${file}" \
      --action=move \
      --addr="${addr}" \
      --newaddr="${newaddr}" \
      --defval="${defval}"
}

jConvert() {
  local file="$1"
  local addr="$2"
  local type="$3"

  "${JEDIT_BIN}" \
      --file="${file}" \
      --action=convert \
      --addr="${addr}" \
      --type="${type}"
}

get_arch() {
  local arch=
  arch="$(uname -m)"

  if [ "${arch}" == "x86_64" ] || [ "${arch}" == "amd64" ] || \
     [ "${arch}" == "x64" ] || [ "${arch}" == "64" ]; then
    arch="amd64"
  elif [ "${arch}" == "i686" ] || [ "${arch}" == "i386" ] || \
       [ "${arch}" == "x86" ] || [ "${arch}" == "32" ]; then
    arch="i386"
  else
    echo "${arch}"
    return 1
  fi

  echo "${arch}"
  return 0
}

export_jedit() {
  local arch=
  arch="$(get_arch; exit $?)"
  if [ $? -ne 0 ]; then
    printf "Warning: Unknown arch: '${arch}', " 1>&2
    arch="amd64"
    echo "use arch='${arch}'" 1>&2
  fi

  JEDIT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
  JEDIT_BIN="${JEDIT_DIR}/jedit-${arch}"

  if [ ! -f "${JEDIT_BIN}" ]; then
    echo "Error: jedit binary does not exist or is not a regular file: '${JEDIT_BIN}'" 1>&2
    return 1
  fi

  if ! "${JEDIT_BIN}" -h > /dev/null; then
    echo "Error: Could not execute jedit binary: '${JEDIT_BIN}'" 1>&2
    return 1
  fi

  export JEDIT_DIR
  export JEDIT_BIN

  export -f jGet
  export -f jSet
  export -f jDel
  export -f jAdd
  export -f jMove
  export -f jConvert

  return 0
}
export_jedit
# nothing more here: return code is important
