#!/usr/bin/env bash
# For jq v1.5

get_arch() {
  local arch=
  arch="$(uname -m)"

  if [ "${arch}" == "x86_64" ] || [ "${arch}" == "amd64" ] || \
     [ "${arch}" == "x64" ] || [ "${arch}" == "64" ]; then
    arch="amd64"
  elif [ "${arch}" == "i686" ] || [ "${arch}" == "i386" ] || \
       [ "${arch}" == "x86" ] || [ "${arch}" == "32" ]; then
    arch="i386"
  else
    echo "${arch}"
    return 1
  fi

  echo "${arch}"
  return 0
}

export_jq() {
  local arch=
  arch="$(get_arch; exit $?)"
  if [ $? -ne 0 ]; then
    printf "Warning: Unknown arch: '${arch}', " 1>&2
    arch="amd64"
    echo "use arch='${arch}'" 1>&2
  fi

  cur_dir="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"

  # For jq 1.5
  if [ "${arch}" == 'i386' ]; then
    JQ_BIN='jq-linux32'
  else
    JQ_BIN='jq-linux64'
  fi

  JQ_BIN="${cur_dir}/${JQ_BIN}"

  if [ ! -f "${JQ_BIN}" ]; then
    echo "Error: jq binary does not exist or is not a regular file: '${JQ_BIN}'" 1>&2
    return 1
  fi

  if ! "${JQ_BIN}" --version > /dev/null; then
    echo "Error: Could not execute jq binary: '${JQ_BIN}'" 1>&2
    return 1
  fi

  jq() { "${JQ_BIN}" "$@"; }

  export JQ_BIN && export -f jq

  return $?
}
export_jq
# nothing more here: return code is important
