#!/usr/bin/env bash
#
# Копирует кастомизаторы из расположения старого приложения (bx24asterisk 2.*)
# в новое (itgrix_bx 3.*) с необходимыми переименованиями.

echo "Migrating customizers from old app version..."

OLD_APP_DIR='/opt/bx24asterisk'
NEW_APP_DIR='/opt/itgrix_bx'

if ! [ -d "${OLD_APP_DIR}" ]; then
  echo "Old version of the app (2.*) was not found at '${OLD_APP_DIR}'"
  exit 1
fi

mkdir -pv "${NEW_APP_DIR}/custom"

if [ -e "${OLD_APP_DIR}/customizer.php" ]; then
  cp -pv "${OLD_APP_DIR}/customizer.php" "${NEW_APP_DIR}/customizer.php"
else
  echo "'${OLD_APP_DIR}/customizer.php' was not found or is not executable"
fi

# в itgrix_bx 3.*, кастомизации телефона пользователя и клиента отдельные,
# поэтому копируем в два файла с новыми именами
if [ -e "${OLD_APP_DIR}/custom/process_phone_number.php" ]; then
  cp -pv "${OLD_APP_DIR}/custom/process_phone_number.php" "${NEW_APP_DIR}/custom/process_user_phone.php"
  cp -pv "${OLD_APP_DIR}/custom/process_phone_number.php" "${NEW_APP_DIR}/custom/process_client_phone.php"
fi
# в itgrix_bx 3.*, get_updated_activity_fields переименовано в customize_activity
if [ -e "${OLD_APP_DIR}/custom/get_updated_activity_fields.php" ]; then
  cp -pv "${OLD_APP_DIR}/custom/get_updated_activity_fields.php" "${NEW_APP_DIR}/custom/customize_activity.php"
fi
# в itgrix_bx 3.*, get_updated_entity_fields переименовано в customize_entity
if [ -e "${OLD_APP_DIR}/custom/get_updated_entity_fields.php" ]; then
  cp -pv "${OLD_APP_DIR}/custom/get_updated_entity_fields.php" "${NEW_APP_DIR}/custom/customize_entity.php"
fi

# эти кастомизаторы остались без изменений - будут скопированы без переименования
unchanged_customizers=( \
  'process_originate_params' \
  'process_record_file_path' \
  'process_task_creation_params' \
  'process_registration_params' \
  'process_final_params' \
  'need_registration' \
)
for cust in "${unchanged_customizers[@]}"; do
  if [ -e "${OLD_APP_DIR}/custom/${cust}.php" ]; then
    cp -pv "${OLD_APP_DIR}/custom/${cust}.php" "${NEW_APP_DIR}/custom/${cust}.php"
  fi
done

echo "Done migrating customizers"
