#!/usr/bin/env bash
{
set -o nounset

SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
readonly SCRIPT_DIR
SCRIPT_FILENAME="$(basename "${BASH_SOURCE[0]}")"
readonly SCRIPT_FILENAME

. "${SCRIPT_DIR}/include.sh"

readonly SCRIPT_NAME="${PRODUCT_NAME}-remove_sudoers"
readonly SCRIPT_VERSION="3.11.10"

LAUNCH_DATETIME_HUMAN="$(date "+${DATETIME_FORMAT_HUMAN}")"
readonly LAUNCH_DATETIME_HUMAN

SCRIPT_USAGE="$(cat <<EOF
Removes sudoers file for ${PRODUCT_NAME}.
RUN ONLY AS root (or via sudo)!

Usage:
  ${SCRIPT_FILENAME} [options]

Options:
  -v
      Show script version.
  --legal
      Show legal information.
  -h, --help
      Show help.
EOF
)"
readonly SCRIPT_USAGE

main() {

  for i in "$@"; do
    case "${i}" in
      -v)
        print_version
        exit
        shift
        ;;
      --legal)
        print_version
        echo
        print_legal
        exit
        shift
        ;;
      -h|--help)
        print_help
        exit
        shift
        ;;
      *) # unknown option
        die_usage "Unknown option: ${i}"
        ;;
    esac
  done

  print_version
  echo "Start at: ${LAUNCH_DATETIME_HUMAN}"

  echo
  if [ -f "${SUDOERS_FILE}" ]; then
    echo "Sudoers file exists: '${SUDOERS_FILE}'"

    echo
    echo "Change mode of sudoers file..."
    chmod 640 "${SUDOERS_FILE}"
    ok_or_die $? "OK" "Failed to change mode of file: '${SUDOERS_FILE}'"

    echo
    echo "Remove sudoers file..."
    rm -r "${SUDOERS_FILE}"
    ok_or_die $? "OK" "Failed to remove file: '${SUDOERS_FILE}'"
  else
    echo "Sudoers file not found: '${SUDOERS_FILE}'"
    echo "Do nothing"
  fi

  echo
  echo -n "Finish " && print_version
}
main "$@"
exit
}
