#!/bin/bash

# Chkconfig: - 98 02
# description: Asterisk and Bitrix connector daemon.
# processname: itgrix_bx

# Для update-rc.d
### BEGIN INIT INFO
# Provides:          itgrix_bx
# Required-Start:    $network $syslog $named $local_fs $remote_fs
# Required-Stop:     $network $syslog $named $local_fs $remote_fs
# Should-Start:      asterisk
# Should-Stop:       asterisk
# Default-Start:     2 3 4 5
# Default-Stop:	     0 1 6
# Short-Description: itgrix_bx
# Description: itgrix_bx
# Processname: itgrix_bx
### END INIT INFO

# Should be replaced during installation
USERNAME=asterisk

NAME="itgrix_bx"
DIR="/opt/itgrix_bx"
LOG_FILE=/var/log/itgrix_bx.log
EXEC=${DIR}"/itgrix_bx >> ${LOG_FILE}"
PID_FILE="/var/run/${NAME}.pid"

printUsage() {
  echo "Usage: service ${NAME} {start|stop|restart|status}"
}

getAndCheckPid() {
  if [ ! -f "${PID_FILE}" ]; then
    return 1
  fi

  local pid=
  pid="$(cat "${PID_FILE}" | awk '/^[0-9]+/{print $0}')"
  echo "${pid}"

  if ! ps -p "${pid}" &>/dev/null; then
    return 1
  fi
  return 0
}

start() {
  local pid=
  pid="$(getAndCheckPid; exit $?)"
  if [ $? -eq 0 ]; then
    echo "${NAME} (pid ${pid}) is already running" >&2
    return 1
  fi

  echo "Starting ${NAME}..." >&2
  if which runuser &>/dev/null; then
    runuser -l "${USERNAME}" -s /bin/sh -c "${EXEC} & echo \$!" > "${PID_FILE}"
  else
    su -c "${EXEC} & echo \$!" "${USERNAME}" > "${PID_FILE}"
  fi

  if [ $? -eq 0 ]; then
    pid="$(getAndCheckPid)"
    echo "${NAME} started (pid ${pid})" >&2
  else
    echo "Failed to start ${NAME}" >&2
    return 1
  fi
}

stop() {
  local pid=
  pid="$(getAndCheckPid; exit $?)"
  if [ $? -ne 0 ]; then
    echo "${NAME} is not running" >&2
    return 1
  fi

  echo "Stopping ${NAME}..." >&2
  kill -15 "${pid}" && rm -f "${PID_FILE}"

  if [ $? -eq 0 ]; then
    echo "${NAME} stopped" >&2
  else
    echo "Failed to stop ${NAME}" >&2
    return 1
  fi
}

restart() {
  stop
  start
}

status() {
  local pid=
  pid="$(getAndCheckPid; exit $?)"
  if [ $? -eq 0 ]; then
    echo "${NAME} (pid ${pid}) is running"
  else
    echo "${NAME} is NOT running"
    return 1
  fi
}

if [ $# -ne 1 ]; then
  printUsage
  exit 1
fi

case "$1" in
  start)
    start
    ;;
  stop)
    stop
    ;;
  restart)
    restart
    ;;
  status)
    status
    ;;
  *)
    printUsage
    exit 1
esac
exit $?
