readonly __SHLIB_ITGRIX_SH=1

# Globals:
#   SCRIPT_NAME
#   SCRIPT_VERSION
#   SCRIPT_USAGE
#   SUDO_CMD

readonly DATETIME_FORMAT_HUMAN="%Y-%m-%d %H:%M:%S%z" # ГГГГ-ММ-ДД ЧЧ:ММ:СС+-ЧЧММ

warn() {
  local msg="$1"
  echo "WARNING: ${msg}"
}

print_version() {
  echo "${SCRIPT_NAME} v${SCRIPT_VERSION}"
}

print_legal() {
cat <<EOF
Copyright:
  © "Razumnie resheniya, LTD", INN 6686029435
  https://itgrix.ru
  https://itgro.ru
  E-mail: bx24@itgro.ru
EOF
}

print_usage() {
  # To be defined in script
  echo "${SCRIPT_USAGE}"
}

print_tech_sup_info() {
cat <<EOF
If you need technical support, please contact us:

  Website:     https://www.itgrix.com; https://itgrix.ru
  Online chat: https://portal.itgro.ru/online/itgrix
  Telegram:    https://teleg.one/itgrix_bot
  E-mail:      bx24@itgro.ru
  Tel.:        +74993508890
               +73432723033
EOF
}

print_help() {
  print_version
  echo
  print_legal
  echo
  print_usage
  echo
  print_tech_sup_info
}

gen_related_opts_error() {
  local options=(${@})
  local msg="Options must be either all specified (nonempty) or all omitted"
  echo "${msg}:$(printf " \"%s\"" "${options[@]}")"
}

die() {
  local msg="$1"

  echo "ERROR: ${msg}"
  echo
  print_tech_sup_info
  echo
  echo "Stop \"${SCRIPT_NAME}\" due to error at: $(date "+${DATETIME_FORMAT_HUMAN}")"
  exit 1
}

ok_or_die() {
  local status=$1
  local ok_msg="$2"
  local die_msg="$3"

  if [ ${status} -ne 0 ]; then
    die "${die_msg}"
  fi

  echo "${ok_msg}"
}

die_usage() {
  local msg=$1
  echo "ERROR: ${msg}"
  echo "Stop \"${SCRIPT_NAME}\" due to error at: $(date "+${DATETIME_FORMAT_HUMAN}")"
  echo
  print_usage
  exit 1
}

get_bin_version() {
  local bin_path="$1"

  local version
  if version="$("${bin_path}" -v 2> /dev/null)"; then
    echo "${version}" | grep -Po "(?<=\s)v.*$" | sed -r 's/v(.+)/\1/'
    return 0
  fi

  echo ""
  return 1
}

escape_version() {
  local version=$1
  # Replace parentheses and spaces with underscore
  echo "${version}" | sed -E 's/[\ \(\)]/_/g'
}

# SUDO_CMD
set_sudo_cmd() {
  SUDO_CMD=''
  local user="$1"

  echo "Check user..."

  if [ "${user}" == 'root' ]; then
    echo "Run as 'root', do not use 'sudo'"
    return 0
  fi

  echo "User is NOT 'root', checking if 'sudo' can be used..."

  if ! command -v sudo &> /dev/null; then
    die "Cannot find 'sudo'. Check if it is installed."
  fi

  if ! sudo -n echo "check_${PRODUCT_NAME}_sudo" &> /dev/null; then
    die "Use of 'sudo' is not configured during ${PRODUCT_NAME} installation"
  fi

  SUDO_CMD='sudo -n'
  echo "OK, use sudo"
}

get_logrotate_status() {
  local log_path="$1"

  local logrotate_status_files=( \
    '/var/lib/logrotate/logrotate.status' \
    '/var/lib/logrotate.status' \
    '/var/lib/logrotate/status' \
  )

  echo "  Check logrotate status file..."
  local status_file=

  local file=
  for file in "${logrotate_status_files[@]}"; do
    echo "    Try file '${file}'..."
    if [ -f "${file}" ]; then
      status_file="${file}"
      break
    else
      echo "    file '${file}' does not exist"
    fi
  done

  if [ "${status_file}" == '' ]; then
    die "Cannot check logrotate status - tried all known status file paths"
  fi
  echo "    OK, use status file '${status_file}'"

  echo "    Check logrotate status file version..."
  local status_file_version=
  status_file_version="$(head -n 1 "${status_file}")"
  if [ "${status_file_version}" != 'logrotate state -- version 2' ]; then
    die "Unknown logrotate status file version: '${status_file_version}'"
  fi
  echo "    OK"

  echo "    Search logrotate status for log file '${log_path}'..."
  local log_status=
  if log_status="$(grep -F "\"${log_path}\"" "${status_file}" 2> /dev/null; exit $?)"; then
    local status_datetime=
    status_datetime="$(echo "${log_status}" | sed -E 's/"[^"]*"\ (.*)/\1/')"
    echo "    Found, last rotated at: '${status_datetime}'"
    return 0
  fi
  return 1
}

# Только для одного лога в конфиге
check_logrotate() {
  local logrotate_config_path="$1"
  local log_path="$2"

  echo "Check logrotate (config: '${logrotate_config_path}'; log: '${log_path}')..."
  if ! logrotate -d "${logrotate_config_path}"; then
    echo
    echo "Detected logrotate error in debug mode, try to force rotation..."
    if get_logrotate_status "${log_path}"; then
      echo "  Try logrotate forced..."
      logrotate -f "${logrotate_config_path}"
      ok_or_die $? "  OK" "Failed to rotate log"
    else
      die "logrotate status file does not contain entry for log file '${log_path}'"
    fi

    echo "Check logrotate again..."
    logrotate -d "${logrotate_config_path}"
  fi
  ok_or_die $? "OK" "logrotate check failed after forced rotation"
}
