readonly __SHLIB_UTILS_SH=1

# Globals:
#   SUDO_CMD (Использование: ${SUDO_CMD:-})

readonly DEFAULT_BOX_WIDTH=80
readonly ESC="$(echo -e "\e")"

#######################################
# Пишет ссылку в виде белого текста на синем фоне. БЕЗ перевода строки.
# Globals:
#   -
# Arguments:
#   1: Ссылка (произвольный текст)
# Outputs:
#   Цветной текст в stdout
#######################################
print_link() {
  local link="$1"
  echo -ne "\e[44;37m${link}\e[m"
}

print_bold() {
  local text="$1"
  echo -ne "\e[1m${text}\e[m"
}

#######################################
# Повторяет заданную строку в указанном количестве. БЕЗ перевода строки.
# Globals:
#   -
# Arguments:
#   1: Строка
#   2: Количество
# Outputs:
#   Текст в stdout
#######################################
print_repeat_unsafe() {
  local str="$1"
  local count="$2"

  local output=
  output="$(printf "%${count}s")"
  echo -n "${output// /${str}}"
}

#######################################
# Рисует горизонтальную границу рамки.
# Globals:
#   DEFAULT_BOX_WIDTH
# Arguments:
#   1: Строка - левый угол
#   2: Символ-заполнитель (между углами)
#   3: Строка - правый угол
#   4: Ширина рамки (если не указана, то DEFAULT_BOX_WIDTH)
# Outputs:
#   Текст в stdout
#######################################
echo_box_h_border() {
  local lcorner="$1"
  local filler="$2"
  local rcorner="$3"
  local width="${4:-${DEFAULT_BOX_WIDTH}}"

  local fill_width=$((width - ${#lcorner} - ${#rcorner}))
  echo "${lcorner}$(print_repeat_unsafe "${filler}" "${fill_width}")${rcorner}"
}

echo_boxed() {
  local lborder="$1"
  local str="$2"
  local rborder="$3"
  local box_width="${4:-${DEFAULT_BOX_WIDTH}}"

  local clean_str=
  clean_str="$(shopt -s extglob; echo "${str//${ESC}\[*([^m])m/}")"

  local diff_len=
  diff_len=$((${box_width} - ${#lborder} - ${#rborder} - 2 + \
              ${#str} - ${#clean_str}))

  printf "${lborder} %-${diff_len}s ${rborder}\n" "${str}"
}

prompt_yesno() {
  local question=$1
  local default=$2 # y|yes or n|no, case insensitive; no default if junk

  local prompt="y/n"

  if echo "${default}" | grep -E -i '^(y|yes)$' > /dev/null ; then
    prompt="Y/n"
    default="y"
  elif echo "${default}" | grep -E -i '^(n|no)$' > /dev/null ; then
    prompt="y/N"
    default="n"
  else
    default=""
  fi

  ans="${default}"

  while true; do
    read -p "${question} [${prompt}] " reply

    if [ "${reply}" == "" ] && [ "${default}" != "" ]; then
      break
    elif echo "${reply}" | grep -E -i '^(y|yes)$' > /dev/null ; then
      ans="y"
      break
    elif echo "${reply}" | grep -E -i '^(n|no)$' > /dev/null ; then
      ans="n"
      break
    else
      continue
    fi
  done

  if [ "${ans}" == "y" ]; then
    return 0
  else
    return 1
  fi
}

prompt_value() {
  local prompt="$1"

  local reply
  read -p "${prompt}: " reply
  echo "${reply}"
}

prompt_nonempty_value() {
  local prompt=$1
  local empty_msg=$2

  local reply=
  while true; do
    read -p "${prompt}: " reply

    if [ "${reply}" == "" ]; then
      echo "${empty_msg}" 1>&2
    else
      echo "${reply}"
      return 0
    fi
  done
}

is_uint() {
  local n="$1"

  local nn="${n//[0-9]/}"
  if [ "${#nn}" -eq 0 ]; then
    return 0
  fi
  return 1
}

int_in_range() {
  local val="$1"
  local lower="$2"
  local upper="$3"

  if [ "${lower}" -le "${val}" ] && [ "${val}" -le "${upper}" ]; then
    return 0
  fi
  return 1
}

is_uint_in_range() {
  local val="$1"
  local lower="$2"
  local upper="$3"

  if ! is_uint "${val}"; then
    return 1
  elif ! int_in_range "${val}" "${lower}" "${upper}"; then
    return 2
  fi
  return 0
}

# Опции адресуем начиная с 1, поэтому:
#   1 <= default <= count
#   1 <= <результат (выбранная опция)> <= count
prompt_numeric_select() {
  local question="$1"
  local preselect_msg="$2"
  local count="$3"
  local default="$4"
  local options=("${@:5}")

  if ! is_uint "${count}" || [ "${#options[@]}" -ne "${count}" ]; then
    return 1
  fi
  if ! is_uint_in_range "${default}" 1 "${count}"; then
    return 2
  fi

  echo "${question}" 1>&2

  local i
  local option_text
  for ((i = 0; i < count; ++i)); do
    option_text="$(printf "[%d] %s\n" "$((i + 1))" "${options[${i}]}")"
    if ((i == default - 1)); then
      print_bold "${option_text} (DEFAULT)" 1>&2
      echo 1>&2
    else
      echo "${option_text}" 1>&2
    fi
  done

  if [ "${#preselect_msg}" -ne 0 ]; then
    echo "${preselect_msg}" 1>&2
  fi

  local reply
  while true; do
    reply="$(prompt_value "Enter selected option number (or press Enter to select default - ${default})" "")"
    if [ "${reply}" == '' ]; then
      break
    fi
    if is_uint_in_range "${reply}" 1 "${count}"; then
      break
    fi
  done

  if [ "${reply}" == '' ]; then
    echo "Nothing selected, use default: [${default}] ${options[$((default - 1))]}" 1>&2
    echo "${default}"
  else
    echo "Selected: [${reply}] ${options[$((reply - 1))]}" 1>&2
    echo "${reply}"
  fi
}

check_dir_write_permissions() {
  local dir="$1"

  local test_file_path
  test_file_path="${dir}/write_permission_test-$(date '+%Y%m%d-%H%M%S')"

  if ! touch "${test_file_path}" &> /dev/null; then
    return 1
  fi
  if ! rm -rf "${test_file_path}" &> /dev/null; then
    return 2
  fi
  return 0
}

# TODO: Вывод текста ошибок в stderr
check_user_group() {
  local user="$1"
  local group="$2"

  if ! getent passwd "${user}" &> /dev/null ; then
    echo "user \"${user}\" doesn't exist"
    return 1
  fi
  if ! getent group "${group}" &> /dev/null ; then
    echo "group \"${group}\" doesn't exist"
    return 2
  fi
  # TODO: &> /dev/null ???
  if ! getent group "${group}" | grep -q "\b${user}\b" ; then
    echo "user \"${user}\" doesn't belong to group \"${group}\""
    return 3
  fi
  return 0
}

exec_service_stop() {
  local name="$1"

  if command -v systemctl &> /dev/null; then
    ${SUDO_CMD:-} systemctl stop "${name}"
  else
    ${SUDO_CMD:-} service "${name}" stop
  fi
}

exec_service_start() {
  local name="$1"

  if command -v systemctl &> /dev/null; then
    ${SUDO_CMD:-} systemctl start "${name}"
  else
    ${SUDO_CMD:-} service "${name}" start
  fi
}

exec_service_status() {
  local name="$1"

  if command -v systemctl &> /dev/null; then
    ${SUDO_CMD:-} systemctl status "${name}"
  else
    ${SUDO_CMD:-} service "${name}" status
  fi
}

stop_service() {
  local name="$1"

  local status_before=
  exec_service_status "${name}" &> /dev/null
  status_before=$?

  local stop_status=
  exec_service_stop "${name}"
  stop_status=$?

  if [ ${stop_status} -ne 0 ] && [ ${status_before} -eq 0 ]; then
    return 1
  fi
  return 0
}

start_service() {
  local name="$1"

  exec_service_start "${name}"
  return $?
}

check_service_status() {
  local name="$1"

  local status_output=
  status_output="$(exec_service_status "${name}" 2>&1; exit $?)"
  if [ $? -ne 0 ]; then
    echo "service status output:" 1>&2
    echo "${status_output}" 1>&2
    return 1
  fi
  return 0
}

systemctl_is_active() {
  systemctl -q is-active $1
}

in_array() {
  local str="$1"
  local arr=("${@:2}")

  for elem in "${arr[@]}"; do
    if [ "${elem}" == "${str}" ]; then
      return 0
    fi
  done
  return 1
}

trim() {
  local str="$1"

  str="$(echo "${str}" | sed -E 's/^\s*(\S)\s*$/\1/')"
  str="$(echo "${str}" | sed -E 's/^\s*(\S.*\S)\s*$/\1/')"
  echo "${str}"
}

bin_is_running() {
  local bin_path="$1"

  if ! (ps -eo command | grep -F "${bin_path}" | grep -v grep) &>/dev/null; then
    return 1
  fi
  return 0
}
