#!/usr/bin/env bash
{
set -o nounset

SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
readonly SCRIPT_DIR
SCRIPT_FILENAME="$(basename "${BASH_SOURCE[0]}")"
readonly SCRIPT_FILENAME

. "${SCRIPT_DIR}/include.sh"

readonly SCRIPT_NAME="${PRODUCT_NAME}-remove"
readonly SCRIPT_VERSION="1.3.4"

LAUNCH_DATETIME_HUMAN="$(date "+${DATETIME_FORMAT_HUMAN}")"
readonly LAUNCH_DATETIME_HUMAN

SCRIPT_USAGE="$(cat <<EOF
Usage:
  ${SCRIPT_FILENAME} [options]

Options:
  --logs=true/false
      Remove (all) log files or not.
      By default, remover will ask.
  -v
      Show script version.
  --legal
      Show legal information.
  -h, --help
      Show help.
EOF
)"
readonly SCRIPT_USAGE

main() {
  local remove_logs='ask'

  for i in "$@"; do
    case "${i}" in
      --logs=*)
        remove_logs="${i#*=}"
        if [ "${remove_logs}" != 'true' ] && [ "${remove_logs}" != 'false' ]; then
          die_usage "Incorrect 'logs' option value: '${remove_logs}'"
        fi
        shift
        ;;
      -v)
        print_version
        exit
        shift
        ;;
      --legal)
        print_version
        echo
        print_legal
        exit
        shift
        ;;
      -h|--help)
        print_help
        exit
        shift
        ;;
      *) # unknown option
        die_usage "Unknown option: ${i}"
        ;;
    esac
  done

  print_version
  echo "Start at: ${LAUNCH_DATETIME_HUMAN}"

  echo
  echo "Stop and remove service ..."
  if command -v systemctl; then
    systemctl stop "${SERVICE_NAME}"
    systemctl disable "${SERVICE_NAME}"
    rm -rf "/etc/systemd/system/${SERVICE_NAME}.service"
  elif command -v chkconfig >/dev/null 2>&1; then
    service "${SERVICE_NAME}" stop
    chkconfig "${SERVICE_NAME}" off
    chkconfig --del "${SERVICE_NAME}"
    rm -rf "/etc/init.d/${SERVICE_NAME}"
  elif command -v update-rc.d; then
    service "${SERVICE_NAME}" stop
    update-rc.d "${SERVICE_NAME}" disable
    rm -rf "/etc/init.d/${SERVICE_NAME}"
  else
    echo "WARNING: Unknown service manager, skip this step."
  fi

  echo
  echo "Remove logrotate config: ${LOGROTATE_CONFIG_PATH} ..."
  rm -rf "${LOGROTATE_CONFIG_PATH}"

  echo
  echo "Remove app dir: ${APP_DIR} ..."
  rm -rf "${APP_DIR}"

  echo
  echo "Logs mode: ${remove_logs}"
  if [ "${remove_logs}" == 'ask' ]; then
    remove_logs='false'
    if prompt_yesno "Do you wish to remove log files (${LOG_PATH}*)?" no ; then
      remove_logs='true'
    fi
  fi
  if [ "${remove_logs}" == 'true' ]; then
    echo "Remove all log files: ${LOG_PATH}* ..."
    rm -rf "${LOG_PATH}"*
  else
    echo "Do NOT remove logs"
  fi

  echo
  echo -n "Finish " && print_version
}
main "$@"
exit
}
